unit CommonfmeOptions_PasswordRequirements;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  OTFEFreeOTFEBase_U, CommonSettings, SDUFrames, SDUFilenameEdit_U,
  OTFEFreeOTFE_VolumeSelect, CommonfmeOptions_Base, Spin64, ExtCtrls;

type
  TfmeOptions_PasswordRequirements = class(TfmeOptions_Base)
    gbPasswordRequirements: TGroupBox;
    lblPassReqMinLength_Chars: TLabel;
    lblPassReqCheckDictionary_DictFile: TLabel;
    ckPassReqMinLength: TCheckBox;
    sePassReqMinLength_Chars: TSpinEdit64;
    ckPassReqCharset: TCheckBox;
    ckPassReqCheckDictionary: TCheckBox;
    fePassReqCheckDictionary_DictFile: TSDUFilenameEdit;
    ckPassReqChPassDifferent: TCheckBox;
    sePassReqChPassDifferent_Pcnt: TSpinEdit64;
    lblPassReqChPassDifferent_Pcnt2: TLabel;
    lblPassReqChPassDifferent_Pcnt1: TLabel;
    ckPassReqKeyboardPattern: TCheckBox;
    ckPassReqRepeatChar: TCheckBox;
    ckPassReqRepeatedWord: TCheckBox;
    ckPassReqSequenceChars: TCheckBox;
    pnlSeparator: TPanel;
    sePassReqCharset_Uppercase: TSpinEdit64;
    sePassReqCharset_Lowercase: TSpinEdit64;
    sePassReqCharset_Numbers: TSpinEdit64;
    sePassReqCharset_Punctuation: TSpinEdit64;
    sePassReqCharset_Space: TSpinEdit64;
    lblPassReqCharset_Uppercase: TLabel;
    lblPassReqCharset_Lowercase: TLabel;
    lblPassReqCharset_Numbers: TLabel;
    lblPassReqCharset_Punctuation: TLabel;
    lblPassReqCharset_Space: TLabel;
    procedure ControlChanged(Sender: TObject);
  private

  public
    OTFEFreeOTFE: TOTFEFreeOTFEBase;

    procedure EnableDisableControls(); override;
    procedure Initialize(); override;
    procedure ReadSettings(config: TSettings); override;
    procedure WriteSettings(config: TSettings); override;
    function  CheckSettings(): boolean; override;
  end;

implementation

{$R *.dfm}

uses
  SDUi18n,
  SDUGeneral,
  PasswordChecks,
  SDUDialogs,
  OTFEFreeOTFEDLL_U;

{$IFDEF _NEVER_DEFINED}
// This is just a dummy const to fool dxGetText when extracting message
// information
// This const is never used; it's #ifdef'd out - SDUCRLF in the code refers to
// picks up SDUGeneral.SDUCRLF
const
  SDUCRLF = ''#13#10;
{$ENDIF}
  
procedure TfmeOptions_PasswordRequirements.ControlChanged(Sender: TObject);
begin
  inherited;
  EnableDisableControls();

end;

procedure TfmeOptions_PasswordRequirements.Initialize();
begin
  SDUCenterControl(gbPasswordRequirements, ccHorizontal);
  SDUCenterControl(gbPasswordRequirements, ccVertical, 25);

  SDUClearDividerPanel(pnlSeparator);

  fePassReqCheckDictionary_DictFile.DefaultExt := DFLT_EXTN_DICTIONARY_WORDLIST;
  fePassReqCheckDictionary_DictFile.OpenDialog.Options := fePassReqCheckDictionary_DictFile.OpenDialog.Options + [ofPathMustExist, ofFileMustExist, ofForceShowHidden];
  fePassReqCheckDictionary_DictFile.OpenDialog.Options := fePassReqCheckDictionary_DictFile.OpenDialog.Options + [ofDontAddToRecent];
  fePassReqCheckDictionary_DictFile.Filter := FILTER_DICTIONARY_WORDLIST_FILES;
  fePassReqCheckDictionary_DictFile.FilterIndex := 0;

end;

procedure TfmeOptions_PasswordRequirements.ReadSettings(config: TSettings);
begin
  ckPassReqMinLength.checked := config.OptPassReqMinLength;
  sePassReqMinLength_Chars.value := config.OptPassReqMinLength_Chars;

  ckPassReqCharset.checked := config.OptPassReqCharset;
  sePassReqCharset_Uppercase.value := config.OptPassReqCharset_Uppercase;
  sePassReqCharset_Lowercase.value := config.OptPassReqCharset_Lowercase;
  sePassReqCharset_Numbers.value := config.OptPassReqCharset_Numbers;
  sePassReqCharset_Punctuation.value := config.OptPassReqCharset_Punctuation;
  sePassReqCharset_Space.value := config.OptPassReqCharset_Space;

  ckPassReqCheckDictionary.checked := config.OptPassReqCheckDictionary;
  fePassReqCheckDictionary_DictFile.Filename := config.OptPassReqCheckDictionary_DictFile;

  ckPassReqChPassDifferent.checked := config.OptPassReqChPassDifferent;
  sePassReqChPassDifferent_Pcnt.Value := config.OptPassReqChPassDifferent_Threshold;

  ckPassReqKeyboardPattern.checked := config.OptPassReqKeyboardPattern;
  // Don't touch - leave as default - config.OptPassReqKeyboardPattern_Length
  ckPassReqRepeatChar.checked := config.OptPassReqRepeatChar;
  // Don't touch - leave as default - config.OptPassReqRepeatChar_Length
  ckPassReqRepeatedWord.checked := config.OptPassReqRepeatedWord;
  ckPassReqSequenceChars.checked := config.OptPassReqSequenceChars;
  // Don't touch - leave as default - config.OptPassReqSequenceChars_Length

end;

procedure TfmeOptions_PasswordRequirements.WriteSettings(config: TSettings);

  function TrueToOneFalseToZero(const x: boolean): integer;
  var
    retval: integer;
  begin
    retval := 0;
    if x then
      begin
      retval := 1;
      end;

    Result:= retval;
  end;

begin
  config.OptPassReqMinLength := ckPassReqMinLength.checked;
  config.OptPassReqMinLength_Chars := sePassReqMinLength_Chars.value;

  config.OptPassReqCharset := ckPassReqCharset.checked;
  config.OptPassReqCharset_Uppercase := sePassReqCharset_Uppercase.value;
  config.OptPassReqCharset_Lowercase := sePassReqCharset_Lowercase.value;
  config.OptPassReqCharset_Numbers := sePassReqCharset_Numbers.value;
  config.OptPassReqCharset_Punctuation := sePassReqCharset_Punctuation.value;
  config.OptPassReqCharset_Space := sePassReqCharset_Space.value;

  config.OptPassReqCheckDictionary := ckPassReqCheckDictionary.checked;
  config.OptPassReqCheckDictionary_DictFile := fePassReqCheckDictionary_DictFile.Filename;

  config.OptPassReqChPassDifferent := ckPassReqChPassDifferent.checked;
  config.OptPassReqChPassDifferent_Threshold := sePassReqChPassDifferent_Pcnt.Value;

  config.OptPassReqKeyboardPattern := ckPassReqKeyboardPattern.checked;
  // Do nothing - leave as default - config.OptPassReqKeyboardPattern_Length := DFLT_OPT_PASSREQKEYBOARDPATTERN_LENGTH;
  config.OptPassReqRepeatChar := ckPassReqRepeatChar.checked;
  // Do nothing - leave as default - config.OptPassReqRepeatChar_Length := DFLT_OPT_PASSREQREPEATCHAR_LENGTH;
  config.OptPassReqRepeatedWord := ckPassReqRepeatedWord.checked;
  config.OptPassReqSequenceChars := ckPassReqSequenceChars.checked;
  // Do nothing - leave as default - config.OptPassReqSequenceChars_Length := DFLT_OPT_PASSREQSEQUENCECHARS_LENGTH;

end;

function TfmeOptions_PasswordRequirements.CheckSettings(): boolean;
var
  allOK: boolean;
begin
  allOK := inherited CheckSettings();

  if allOK then
    begin
    if ckPassReqCharset.checked then
      begin
      // At least one option must be selected
      if not(
             (sePassReqCharset_Uppercase.value > 0) or
             (sePassReqCharset_Lowercase.value > 0) or
             (sePassReqCharset_Numbers.value > 0) or
             (sePassReqCharset_Punctuation.value > 0) or
             (sePassReqCharset_Space.value > 0)
            ) then
        begin
        SDUMessageDlg(
                      _('If passwords must contain certain types of characters, please select which characters must be represented by the password.'),
                      mtError
                     );
        allOK := FALSE;
        end;
      end;
    end;

  if allOK then
    begin
    if ckPassReqCheckDictionary.checked then
      begin
      if (fePassReqCheckDictionary_DictFile.Filename = '') then
        begin
        SDUMessageDlg(
                      _('If passwords are to be checked against a dictionary, the dictionary file to use must be specified.'),
                      mtError
                     );
        allOK := FALSE;
        end
      else if not(FileExists(fePassReqCheckDictionary_DictFile.Filename)) then
        begin
        SDUMessageDlg(
                      _('If passwords are to be checked against a dictionary, the dictionary file specified must be available.'),
                      mtError
                     );
        allOK := FALSE;
        end;
      end;
    end;

  if allOK then
    begin
    if ckPassReqChPassDifferent.checked then
      begin
      if (
          (sePassReqChPassDifferent_Pcnt.Value < 1) or
          (sePassReqChPassDifferent_Pcnt.Value > 100)
         ) then
        begin
        SDUMessageDlg(
                      _('Please specify a threshold difference between old and new passwords of between 1 - 100'),
                      mtError
                     );
        allOK := FALSE;
        end;
      end;
    end;

  Result := allOK;
end;

procedure TfmeOptions_PasswordRequirements.EnableDisableControls();
begin
  inherited;

  SDUEnableControl(sePassReqMinLength_Chars, ckPassReqMinLength.checked);
  SDUEnableControl(lblPassReqMinLength_Chars, ckPassReqMinLength.checked);

  SDUEnableControl(lblPassReqCharset_Uppercase, ckPassReqCharset.checked);
  SDUEnableControl(lblPassReqCharset_Lowercase, ckPassReqCharset.checked);
  SDUEnableControl(lblPassReqCharset_Numbers, ckPassReqCharset.checked);
  SDUEnableControl(lblPassReqCharset_Punctuation, ckPassReqCharset.checked);
  SDUEnableControl(lblPassReqCharset_Space, ckPassReqCharset.checked);
  SDUEnableControl(sePassReqCharset_Uppercase, ckPassReqCharset.checked);
  SDUEnableControl(sePassReqCharset_Lowercase, ckPassReqCharset.checked);
  SDUEnableControl(sePassReqCharset_Numbers, ckPassReqCharset.checked);
  SDUEnableControl(sePassReqCharset_Punctuation, ckPassReqCharset.checked);
  SDUEnableControl(sePassReqCharset_Space, ckPassReqCharset.checked);

  SDUEnableControl(lblPassReqCheckDictionary_DictFile, ckPassReqCheckDictionary.checked);
  SDUEnableControl(fePassReqCheckDictionary_DictFile, ckPassReqCheckDictionary.checked);

  SDUEnableControl(lblPassReqChPassDifferent_Pcnt1, ckPassReqChPassDifferent.checked);
  SDUEnableControl(lblPassReqChPassDifferent_Pcnt2, ckPassReqChPassDifferent.checked);
  SDUEnableControl(sePassReqChPassDifferent_Pcnt, ckPassReqChPassDifferent.checked);

end;

END.

